<?php
/* This file reads Firefox bookmarks from foxmarks.json and displays it as HTML.
 * created by Stefan Macke 
 * http://blog.stefan-macke.com/2008/05/11/firefox-bookmarks-aus-foxmarks-anzeigen-json-datei/ 
*/

// please enter the complete path to foxmarks.json here
$foxmarksFile = "foxmarks.json";

?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" lang="en" xml:lang="en">
<head>
    <meta http-equiv="Content-Type" content="text/html;charset=UTF-8" />
    <title>Stefans Bookmarks</title>
<link rel="stylesheet" type="text/css" media="all" href="foxmarks.css" />
    <script type='text/javascript'>
        function toggleFolderView (szDivID) {
            var obj = document.layers ? document.layers['folder'+szDivID] : document.getElementById ? document.getElementById('folder'+szDivID).style : document.all['folder'+szDivID].style; 
            obj.display = (obj.display == 'block' ? 'none' : 'block'); 
            var obj_title = document.layers ? document.layers['foldertitle'+szDivID] : document.getElementById ?  document.getElementById('foldertitle'+szDivID).style : document.all['foldertitle'+szDivID].style;
            obj_title.backgroundImage = (obj_title.backgroundImage == "url(folder_open.gif)" ? "url(folder_closed.gif)" : "url(folder_open.gif)");
        }
    </script>               

</head>

<body>
<?php
if (!is_readable($foxmarksFile))
{
	echo "Foxmarks file is not readable! Please check the file permissions.";
}
else
{
	$content = file_get_contents($foxmarksFile);
	$json = json_decode($content);
	
	if (($json != null) && ($json->commands != null))
	{
	    $nodes = array();
	    foreach ($json->commands as $node)
	        $nodes[] = $node;
	    $tree = array();
	    createTree($tree, $nodes, "ROOT");
	    printTree($tree, 0);
	}
}

function createTree(&$tree, $nodes, $rootNodeId)
{
    for ($i = 0; $i < count($nodes); $i++)
    {
        if ($nodes[$i]->args->pnid == $rootNodeId)
        {
            $id = $nodes[$i]->nid;
            $type = $nodes[$i]->args->ntype;
            $name = $nodes[$i]->args->name;
            switch ($type)
            {
                case "folder":
                    $folder = array("id" => $id, "type" => $type, "name" => $name, "children" => array());
                    createTree($folder['children'], $nodes, $id);
                    $tree[] = $folder;
                    break;
                case "bookmark":
                    $url = $nodes[$i]->args->url;
                    $icon = $nodes[$i]->args->icon;
                    $tree[] = array("id" => $id, "type" => $type, "name" => $name, "url" => $url, "icon" => $icon);
                    break;
                default:
                    break;
            }
        }
    }
}

function printTree($tree, $indent, $folderId = "")
{
    echo str_repeat("\t", $indent) . "<ul";
    if ($folderId != "")
    {
        echo ' id="folder' . $folderId . '"';
    }
    else
    {
        echo ' id="root"';
    }
    echo ">\n";
    foreach ($tree as $node)
    {
        switch ($node['type'])
        {
            case "folder":
                echo str_repeat("\t", $indent + 1) . '<li class="folder" id="foldertitle' . $node['id'] . '"><a href="JavaScript:toggleFolderView(\'' . $node['id'] . '\')">' . htmlspecialchars($node['name']) . "</a>\n";
                printTree($node['children'], $indent + 1, $node['id']);
                echo str_repeat("\t", $indent + 1) . "</li><!-- " . $node['name'] . "-->\n";
                break;
            case "bookmark":
                $name = htmlspecialchars($node['name']);
                $url = htmlentities($node['url']);
                $name = trim($name) != "" ? $name : $url;
                $icon = "";
                if ($node['icon'] != "")
                {
					$icon = '<img alt="Icon" src="' . $node['icon'] . '" />&nbsp;';
				}
                echo str_repeat("\t", $indent + 1) . '<li class="bookmark"><a href="' . $url . '" title="' . $name . '">' . $icon . $name . "</a></li>\n";
                break;
            default:
                break;
        }
    }
    echo str_repeat("\t", $indent) . "</ul>\n";
}
?>
</body>
</html>
